# -*- coding: utf-8 -*-
from collective.nitf import _
from collective.nitf.defaults import genre_default_value
from collective.nitf.defaults import urgency_default_value
from plone.app.textfield import RichText
from plone.directives import form
from plone.supermodel import model
from zope import schema
from zope.interface import Interface


class INITFLayer(Interface):
    """ A layer specific for this add-on product.
    """


class INITF(model.Schema):

    """A News Article based on the News Industry Text Format specification."""

    # title = schema.TextLine()
    # nitf/head/title and nitf/body/body.head/hedline/hl1

    form.order_before(subtitle='IDublinCore.title')
    subtitle = schema.TextLine(
        # nitf/body/body.head/hedline/hl2
        title=_(u'Subtitle'),
        description=_(u'help_subtitle',
                      default=u'A subordinate headline for the article.'),
        default=u'',
        missing_value=u'',
        required=False,
    )

    byline = schema.TextLine(
        # nitf/body/body.head/byline/person
        title=_(u'Author'),
        default=u'',
        missing_value=u'',
        required=False,
    )

    location = schema.TextLine(
        # nitf/body/body.head/dateline/location
        title=_(u'Location'),
        description=_(
            u'help_location',
            default=u'Event location. Where an event took place '
                    u'(as opposed to where the story was written).',
        ),
        default=u'',
        missing_value=u'',
        required=False,
    )

    text = RichText(
        # nitf/body/body.content
        title=_(u'Body text'),
        required=False,
    )

    model.fieldset('categorization', fields=['genre', 'urgency'])

    genre = schema.Choice(
        # nitf/head/tobject/tobject.property/@tobject.property.type
        title=_(u'Genre'),
        description=_(
            u'help_genre',
            default=u'Describes the nature, journalistic or '
                    u'intellectual characteristic of a news '
                    u'object, not specifically its content.',
        ),
        vocabulary=u'collective.nitf.AvailableGenres',
        defaultFactory=genre_default_value
    )

    urgency = schema.Choice(
        # nitf/head/docdata/urgency/@ed-urg
        title=_(u'Urgency'),
        description=_(u'help_urgency',
                      default=u'News importance.'),
        vocabulary=u'collective.nitf.Urgencies',
        defaultFactory=urgency_default_value
    )



# -*- coding: utf-8 -*-
from collective.nitf import _
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


PROJECTNAME = 'collective.nitf'

# this cames from http://cv.iptc.org/newscodes/genre/
GENRES = SimpleVocabulary([
    SimpleTerm(value=u'Actuality',
               title=_(u'Actuality')),
    # Definition: The object contains the recording of the event.
    SimpleTerm(value=u'Advice',
               title=_(u'Advice')),
    # Definition: The object contains advice, typically letters and answers
    # about personal problems, that are publishable.
    SimpleTerm(value=u'Almanac',
               title=_(u'Almanac')),
    # Definition: List of data, including birthdays of famous people and items
    # of historical significance, for a given day.
    SimpleTerm(value=u'Analysis',
               title=_(u'Analysis')),
    # Definition: The object contains data and conclusions drawn by a
    # journalist who has researched the story in depth.
    SimpleTerm(value=u'Anniversary',
               title=_(u'Anniversary')),
    # Definition: Stories about the anniversary of some important event that
    # took place in recent history, usually bringing a short review of the
    # event itself.
    SimpleTerm(value=u'Archive material',
               title=_(u'Archive material')),
    # Definition: The object contains material distributed previously that has
    # been selected from the originator's archives.
    SimpleTerm(value=u'Background',
               title=_(u'Background')),
    # Definition: The object provides some scene setting and explanation for
    # the event being reported.
    SimpleTerm(value=u'Current',
               title=_(u'Current')),
    # Definition: The object content is about events taking place at the time
    # of the report.
    SimpleTerm(value=u'Curtain Raiser',
               title=_(u'Curtain Raiser')),
    # Definition: The object contains information about the staging and outcome
    # of an immediately upcoming event.
    SimpleTerm(value=u'Daybook',
               title=_(u'Daybook')),
    # Definition: Items filed on a regular basis that are lists of upcoming
    # events with time and place, designed to inform others of events for
    # planning purposes.
    SimpleTerm(value=u'Exclusive',
               title=_(u'Exclusive')),
    # Definition: Information content, in any form, that is unique to a
    # specific information provider.
    SimpleTerm(value=u'Feature',
               title=_(u'Feature')),
    # Definition: The object content is about a particular event or individual
    # that may not be significant to the current breaking news.
    SimpleTerm(value=u'Fixture',
               title=_(u'Fixture')),
    # Definition: The object contains data that occurs often and predictably.
    SimpleTerm(value=u'Forecast',
               title=_(u'Forecast')),
    # Definition: The object contains opinion as to the outcome of a future
    # event.
    SimpleTerm(value=u'From the Scene',
               title=_(u'From the Scene')),
    # Definition: The object contains a report from the scene of an event.
    SimpleTerm(value=u'History',
               title=_(u'History')),
    # Definition: The object content is based on previous rather than current
    # events.
    SimpleTerm(value=u'Horoscope',
               title=_(u'Horoscope')),
    # Definition: Astrological forecasts
    SimpleTerm(value=u'Interview',
               title=_(u'Interview')),
    # Definition: The object contains a report of a dialogue with a news source
    # that gives it significant voice (includes Q and A).
    SimpleTerm(value=u'Music',
               title=_(u'Music')),
    # Definition: The object contains music alone.
    SimpleTerm(value=u'Obituary',
               title=_(u'Obituary')),
    # Definition: The object contains a narrative about an individual's life
    # and achievements for publication after his or her death.
    SimpleTerm(value=u'Opinion',
               title=_(u'Opinion')),
    # Definition: The object contains an editorial comment that reflects the
    # views of the author.
    SimpleTerm(value=u'Polls and Surveys',
               title=_(u'Polls and Surveys')),
    # Definition: The object contains numeric or other information produced as
    # a result of questionnaires or interviews.
    SimpleTerm(value=u'Press Release',
               title=_(u'Press Release')),
    # Definition: The object contains promotional material or information
    # provided to a news organisation.
    SimpleTerm(value=u'Press-Digest',
               title=_(u'Press-Digest')),
    # Definition: The object contains an editorial comment by another medium
    # completely or in parts without significant journalistic changes.
    SimpleTerm(value=u'Profile',
               title=_(u'Profile')),
    # Definition: The object contains a description of the life or activity of
    # a news subject (often a living individual).
    SimpleTerm(value=u'Program',
               title=_(u'Program')),
    # Definition: A news item giving lists of intended events and time to be
    # covered by the news provider. Each program covers a day, a week, a month
    # or a year. The covered period is referenced as a keyword.
    SimpleTerm(value=u'Question and Answer Session',
               title=_(u'Question and Answer Session')),
    # Definition: The object contains the interviewer and subject questions and
    # answers.
    SimpleTerm(value=u'Quote',
               title=_(u'Quote')),
    # Definition: The object contains a one or two sentence verbatim in direct
    # quote.
    SimpleTerm(value=u'Raw Sound',
               title=_(u'Raw Sound')),
    # Definition: The object contains unedited sounds.
    SimpleTerm(value=u'Response to a Question',
               title=_(u'Response to a Question')),
    # Definition: The object contains a reply to a question.
    SimpleTerm(value=u'Results Listings and Statistics',
               title=_(u'Results Listings and Statistics')),
    # Definition: The object contains alphanumeric data suitable for
    # presentation in tabular form.
    SimpleTerm(value=u'Retrospective',
               title=_(u'Retrospective')),
    # Definition: The object contains material that looks back on a specific
    # (generally long) period of time such as a season, quarter, year or
    # decade.
    SimpleTerm(value=u'Review',
               title=_(u'Review')),
    # Definition: The object contains a critique of a creative activity or
    # service (for example a book, a film or a restaurant).
    SimpleTerm(value=u'Scener',
               title=_(u'Scener')),
    # Definition: The object contains a description of the event circumstances.
    SimpleTerm(value=u'Side bar and supporting information',
               title=_(u'Side bar and supporting information')),
    # Definition: The object contains a related story that provides additional
    # insight into the news event being reported.
    SimpleTerm(value=u'Special Report',
               title=_(u'Special Report')),
    # Definition: In-depth examination of a single subject requiring extensive
    # research and usually presented at great length, either as a single item
    # or as a series of items.
    SimpleTerm(value=u'Summary',
               title=_(u'Summary')),
    # Definition: The object contains a single item synopsis of a number of
    # news stories (generally unrelated).
    SimpleTerm(value=u'Synopsis',
               title=_(u'Synopsis')),
    # Definition: The object contains a condensed version of a single news
    # item.
    SimpleTerm(value=u'Text only',
               title=_(u'Text only')),
    # Definition: The object contains a transcription of text.
    SimpleTerm(value=u'Transcript and Verbatim',
               title=_(u'Transcript and Verbatim')),
    # Definition: The object contains a word for word report of a discussion or
    # briefing without significant journalistic intervention.
    SimpleTerm(value=u'Update',
               title=_(u'Update')),
    # Definition: The object contains an intraday snapshot (as for electronic
    # services) of a single news subject.
    SimpleTerm(value=u'Voicer',
               title=_(u'Voicer')),
    # Definition: The object contains only voice.
    SimpleTerm(value=u'Wrap',
               title=_(u'Wrap')),
    # Definition: The object contains a complete summary of the event.
    SimpleTerm(value=u'Wrapup',
               title=_(u'Wrapup')),
    # Definition: The object contains a recap of a running story (such as the
    # end of the day).
])

HIGH = 1
NORMAL = 5
LOW = 8

URGENCIES = SimpleVocabulary([
    SimpleTerm(value=HIGH, title=_(u'High')),
    SimpleTerm(value=NORMAL, title=_(u'Normal')),
    SimpleTerm(value=LOW, title=_(u'Low')),
])

# defaults
DEFAULT_GENRE = u'Current'
DEFAULT_SECTION = _(u'General')
DEFAULT_URGENCY = NORMAL

# Cycle2 JS resources used by the package
JS_RESOURCES = (
    '++resource++collective.js.cycle2/jquery.cycle2.min.js',
    '++resource++collective.js.cycle2/jquery.cycle2.carousel.min.js',
    '++resource++collective.js.cycle2/jquery.cycle2.swipe.min.js',
)



# -*- coding: utf-8 -*-
"""Tests in this module are not executed in Plone 5 as the documentbyline
viewlet is not visible by default. We need to find out how to fix that.

For more information on how to test viewlets, see:
http://docs.plone.org/develop/plone/views/viewlets.html
"""
from collective.nitf.interfaces import INITFLayer
from collective.nitf.testing import INTEGRATION_TESTING
from collective.nitf.testing import IS_PLONE_5
from plone import api
from Products.Five.browser import BrowserView as View
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager

import unittest

if IS_PLONE_5:
    def test_suite():
        return unittest.TestSuite()


class DocumentBylineViewletTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        alsoProvides(self.request, INITFLayer)

        self.manager = 'plone.belowcontenttitle'
        self.viewlet = 'plone.belowcontenttitle.documentbyline'

        with api.env.adopt_roles(['Manager']):
            self.n1 = api.content.create(
                self.portal, 'collective.nitf.content', 'n1')

    def _get_viewlet_manager(self, context, request=None, name=None):
        assert name is not None
        if request is None:
            request = self.request
        view = View(context, request)
        manager = getMultiAdapter(
            (context, request, view), IViewletManager, name)
        return manager

    def _get_viewlet(self, context, manager, name):
        manager = self._get_viewlet_manager(context, name=manager)
        manager.update()
        viewlet = [v for v in manager.viewlets if v.__name__ == name]
        assert len(viewlet) == 1
        return viewlet[0]

    def test_non_overrides(self):
        context = self.portal
        documentbyline = self._get_viewlet(context, self.manager, self.viewlet)
        self.assertFalse(hasattr(documentbyline, '_search_member_by_name'))

    def test_overrides(self):
        context = self.n1
        documentbyline = self._get_viewlet(context, self.manager, self.viewlet)
        self.assertTrue(hasattr(documentbyline, '_search_member_by_name'))

    def test_render_empty_byline(self):
        documentbyline = self._get_viewlet(self.n1, self.manager, self.viewlet)

        self.assertNotIn(
            u'<span class="documentAuthor">', documentbyline.render())

    def test_render_non_existent_user(self):
        documentbyline = self._get_viewlet(self.n1, self.manager, self.viewlet)

        self.n1.byline = u'Keith Moon'
        self.assertIn(
            u'<span class="documentAuthor">', documentbyline.render())
        self.assertNotIn(u'property="rnews:author"', documentbyline.render())
        self.assertIn(u'Keith Moon', documentbyline.render())

    def test_render_existent_user(self):
        api.user.create(
            email='foo@bar.com',
            username='keith-moon',
            properties=dict(fullname=u'Keith Moon'),
        )

        documentbyline = self._get_viewlet(self.n1, self.manager, self.viewlet)

        self.n1.byline = u'Keith Moon'
        self.assertIn(
            u'<span class="documentAuthor">', documentbyline.render())
        self.assertIn(
            u'<a href="http://nohost/plone/author/keith-moon" property="rnews:author">Keith Moon</a>',
            documentbyline.render(),
        )



# -*- coding: utf-8 -*-
"""Tests in this module are executed only if collective.cover is installed."""
from collective.nitf.testing import HAS_COVER
from collective.nitf.testing import INTEGRATION_TESTING
from lxml import etree
from mock import Mock
from plone import api

import unittest

if HAS_COVER:
    from collective.cover.tests.base import TestTileMixin
    from collective.nitf.tiles.nitf import INITFTile
    from collective.nitf.tiles.nitf import NITFTile
else:
    class TestTileMixin:
        pass

    def test_suite():
        return unittest.TestSuite()


class NITFTileTestCase(TestTileMixin, unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        super(NITFTileTestCase, self).setUp()
        self.tile = NITFTile(self.cover, self.request)
        self.tile.__name__ = u'collective.nitf'
        self.tile.id = u'test'

    @unittest.expectedFailure  # FIXME: raises BrokenImplementation
    def test_interface(self):
        self.interface = INITFTile
        self.klass = NITFTile
        super(NITFTileTestCase, self).test_interface()

    def test_default_configuration(self):
        self.assertTrue(self.tile.is_configurable)
        self.assertTrue(self.tile.is_editable)
        self.assertTrue(self.tile.is_droppable)

    def test_accepted_content_types(self):
        self.assertEqual(self.tile.accepted_ct(), ['collective.nitf.content'])

    def test_render_empty(self):
        msg = u'Drag&amp;drop a News Article here.'

        self.tile.is_compose_mode = Mock(return_value=True)
        self.assertIn(msg, self.tile())

        self.tile.is_compose_mode = Mock(return_value=False)
        self.assertNotIn(msg, self.tile())

    def test_render(self):
        kwargs = dict(title=u'foo', subtitle=u'bar', section=u'baz')
        with api.env.adopt_roles(['Manager']):
            n1 = api.content.create(
                self.portal, 'collective.nitf.content', 'n1', **kwargs)

        self.tile.populate_with_object(n1)
        html = etree.HTML(self.tile())
        self.assertIn('foo', html.xpath('//h2/a/text()'))
        self.assertIn('bar', html.xpath('//p/text()'))
        self.assertIn('baz', html.xpath('//p/text()'))
        self.assertTrue(html.xpath('//time'))

    def test_alt_atribute_present_in_image(self):
        # https://github.com/collective/collective.nitf/issues/152
        from collective.nitf.testing import FRACTAL
        from collective.nitf.tests.api_hacks import set_image_field

        with api.env.adopt_roles(['Manager']):
            n1 = api.content.create(
                self.portal, 'collective.nitf.content', title='Lorem ipsum')

        image = api.content.create(n1, 'Image', title='Neque porro')
        set_image_field(image, FRACTAL, 'image/jpeg')

        self.tile.populate_with_object(n1)
        html = etree.HTML(self.tile())
        # title of the news article is used as alt attribute
        self.assertIn('Lorem ipsum', html.xpath('//img/@alt'))

    def test_render_deleted_object(self):
        # https://github.com/collective/collective.nitf/issues/154
        with api.env.adopt_roles(['Manager']):
            n1 = api.content.create(
                self.portal, 'collective.nitf.content', title='Lorem ipsum')

        self.tile.populate_with_object(n1)
        api.content.delete(n1)
        # tile's data is cached; reinstantiate it
        self.tile = self.cover.restrictedTraverse('@@collective.nitf/test')
        html = etree.HTML(self.tile())
        # some metadata is still present
        self.assertIn('Lorem ipsum', html.xpath('//h2/a/text()'))
        self.assertFalse(html.xpath('//time'))  # date is ignored



# -*- coding: utf-8 -*-
from collective.nitf.interfaces import INITF
from plone.app.textfield.interfaces import ITransformer
from plone.dexterity.content import Container
from plone.indexer import indexer
from Products.CMFPlone.utils import safe_unicode
from zope.deprecation import deprecation
from zope.interface import implementer


@implementer(INITF)
class NITF(Container):

    """A News Article based on the News Industry Text Format specification."""

    def is_empty(self):
        """Return True if the container has no files nor links inside.
        """
        content_filter = {'portal_type': ['File', 'Link']}
        return not self.listFolderContents(content_filter)

    def get_images(self):
        """Return a list of images contained in the News Article."""
        content_filter = {'portal_type': 'Image'}
        return self.listFolderContents(content_filter)

    # FIXME: @property
    def image(self):
        """Return the first contained image."""
        images = self.get_images()
        return images[0] if len(images) > 0 else None

    def media_caption(self):
        """Return the description of the first contained image."""
        try:
            return self.image().Description()
        except AttributeError:
            return u''

    def media_producer(self):
        """Return the author of the first contained image."""
        try:
            return self.image().Rights()
        except AttributeError:
            return u''

    # XXX: emulate News Item methods and
    #      deal with issues created by our fake image field

    @deprecation.deprecate('getImage() is deprecated; use image().')
    def getImage(self):
        return self.image()

    @deprecation.deprecate('imageCaption() is deprecated; use media_caption.')
    def imageCaption(self):
        return self.media_caption()

    image_thumb = image

    def tag(self, **kwargs):
        try:
            scales = self.image().restrictedTraverse('@@images')
            return scales.tag('image', **kwargs)
        except AttributeError:
            return None


@indexer(INITF)
def textIndexer(obj):
    """SearchableText contains id, title, subtitle, abstract, author and body
    text as plain text.
    """
    transformer = ITransformer(obj)
    text = obj.text
    if text:
        text = transformer(obj.text, 'text/plain')

    searchable_text = [safe_unicode(entry) for entry in (
        obj.id,
        obj.Title(),
        obj.subtitle,
        obj.Description(),
        obj.byline,
        text,
        obj.location,
    ) if entry]

    return u' '.join(searchable_text)



# -*- coding: utf-8 -*-
from collective.nitf.testing import INTEGRATION_TESTING
from plone import api

import unittest


class PermissionsTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        with api.env.adopt_roles(['Manager']):
            self.n1 = api.content.create(
                self.portal, 'collective.nitf.content', 'n1')

    def test_add_permissions(self):
        permission = 'collective.nitf: Add News Article'
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        expected = ['Contributor', 'Manager', 'Owner', 'Site Administrator']
        self.assertListEqual(roles, expected)

    # TODO: find a better way to test this
    def test_owner_permissions(self):
        # the owner can add Images, Files and Links
        # Manager is the owner in this context
        permissions = self.n1.permissionsOfRole('Manager')
        permissions = [p['name'] for p in permissions if p['selected']]
        self.assertIn('ATContentTypes: Add Image', permissions)
        self.assertIn('ATContentTypes: Add File', permissions)
        self.assertIn('ATContentTypes: Add Link', permissions)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from collective.nitf.browser import View
from plone.uuid.interfaces import IUUID

import mimetypes


class NITF(View):

    """Shows news article in NITF XML format."""

    def update(self):
        self.context = aq_inner(self.context)
        self.uuid = IUUID(self.context)

    def _get_mediatype(self, mimetype):
        """ Return one of the possible values of the media-type controlled
        vocabulary.
        """
        # 'data' and 'other' are also part of the controlled vocabulary; we
        # are not going to use 'data'
        vocabulary = ['text', 'audio', 'image', 'video', 'application']
        for i in vocabulary:
            if mimetype.find(i) != -1:
                return i

        return 'other'

    MEDIA = """
<media media-type="{0}">
    <media-reference mime-type="{1}" source="{2}" alternate-text="{3}"{4}{5} />
    <media-caption>{6}</media-caption>
</media>"""

    def get_media(self):
        """ Return a list of object brains inside the NITF object.
        """
        media = []
        # XXX: we could honor original order calling the get_media() method in
        # View; how can we do that?
        results = self.get_images() + self.get_files()
        for r in results:
            obj = r.getObject()
            source = obj.absolute_url()
            (mimetype, encoding) = mimetypes.guess_type(source)
            # if no mime type is detected, result is None; we must change it
            mimetype = mimetype and mimetype or ''
            mediatype = self._get_mediatype(mimetype)
            alternate_text = obj.Title()
            caption = obj.Description()

            # we only include height and/or width if we have a value for them
            try:
                height, width = obj.getHeight(), obj.getWidth()
                height = ' height="{0}"'.format(height) if height else ''
                width = ' width="{0}"'.format(width) if width else ''
            except AttributeError:  # FIXME: Dexterity
                width = height = ''

            m = self.MEDIA.format(
                mediatype, mimetype, source, alternate_text, height, width, caption)
            media.append(m)

        return media



# -*- coding: utf-8 -*-
"""Functions that will be called when the add form of the content type
is loaded to determine the default values of a field.
"""
from collective.nitf.controlpanel import INITFSettings
from plone import api


def genre_default_value():
    """Return the default value for the genre field as defined in the
    control panel configlet.
    """
    record = INITFSettings.__identifier__ + '.default_genre'
    return api.portal.get_registry_record(record)


def urgency_default_value():
    """Return the default value for the urgency field as defined in
    the control panel configlet.
    """
    record = INITFSettings.__identifier__ + '.default_urgency'
    return api.portal.get_registry_record(record)



# -*- coding: utf-8 -*-
from collective.nitf.config import PROJECTNAME
from plone import api


def remove_tile(portal):
    tiles = api.portal.get_registry_record('plone.app.tiles')
    if u'collective.nitf' in tiles:
        tiles.remove(u'collective.nitf')


def uninstall(portal, reinstall=False):
    if not reinstall:
        remove_tile(portal)
        profile = 'profile-{0}:uninstall'.format(PROJECTNAME)
        setup_tool = api.portal.get_tool('portal_setup')
        setup_tool.runAllImportStepsFromProfile(profile)
        return 'Ran all uninstall steps.'



# coding: utf-8
from collective.nitf import _
from collective.nitf.controlpanel import INITFSettings
from plone.directives import form
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from zope import schema
from zope.component import getUtility
from zope.interface import provider


@provider(form.IFormFieldProvider)
class ISection(model.Schema):

    """Behavior interface to make a content type support sections."""

    model.fieldset('categorization', fields=['section'])
    form.order_before(section='genre')

    # nitf/head/pubdata/@position.section
    section = schema.Choice(
        title=_(u'Section'),
        description=_(
            u'help_section',
            default=u'Named section where the item will appear.',
        ),
        vocabulary=u'collective.nitf.AvailableSections',
    )


@form.default_value(field=ISection['section'])
def section_default_value(data):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(INITFSettings)
    return settings.default_section



# -*- coding:utf-8 -*-
from collective.nitf.config import PROJECTNAME
from collective.nitf.logger import logger
from plone import api

import logging


def apply_profile(context, logger=None):
    """Apply upgrade profile; this includes:

    - remove character counter resources from CSS and JS registries
    - remove character counter control panel records from registry
    - rename galleria view in News Article content type
    - cook CSS and JS resources
    """
    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    profile = 'profile-collective.nitf.upgrades.v2000:default'
    setup = api.portal.get_tool('portal_setup')
    setup.runAllImportStepsFromProfile(profile)

    portal_css = api.portal.get_tool('portal_css')
    portal_css.cookResources()
    logger.info(u'CSS resources cooked')
    portal_js = api.portal.get_tool('portal_javascripts')
    portal_js.cookResources()
    logger.info(u'JS resources cooked')


def update_layouts(context, logger=None):
    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    # update existing objects
    logger.info(u'Updating layout of News Articles')
    catalog = api.portal.get_tool('portal_catalog')
    results = catalog(portal_type='collective.nitf.content')
    logger.info(u'{0} News Articles found'.format(len(results)))
    i = 0
    for item in results:
        obj = item.getObject()
        if obj.getLayout() == 'nitf_galleria':
            obj.setLayout('slideshow_view')
            i += 1
        logger.info(u'{0} News Articles updated'.format(i))


def install_new_dependencies(context, logger=None):
    logger = logging.getLogger(PROJECTNAME)
    dependencies = ('collective.js.cycle2',)
    qi = api.portal.get_tool('portal_quickinstaller')
    for p in dependencies:
        if not qi.isProductInstalled(p):
            qi.installProducts([p])
            logger.info(u'{0} installed'.format(p))


def update_configlet(setup_tool):
    """Update control panel configlet."""
    profile = 'profile-{0}:default'.format(PROJECTNAME)
    setup_tool.runImportStepFromProfile(profile, 'controlpanel')
    logger.info('Control panel configlet updated.')



# -*- coding: utf-8 -*-

from collective.nitf.testing import FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import os
import unittest


dirname = os.path.dirname(__file__)
files = os.listdir(dirname)
tests = [f for f in files if f.startswith('test_') and f.endswith('.txt')]


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(doctest.DocFileSuite(t), layer=FUNCTIONAL_TESTING)
        for t in tests
    ])
    return suite



# -*- coding: utf-8 -*-
from collective.nitf.config import PROJECTNAME
from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer

import logging


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Do not show on Plone's list of installable profiles."""
        return [
            u'collective.nitf:uninstall',
        ]


class Empty:
    pass


def add_catalog_indexes(context, logger=None):
    """ Method to add our wanted indexes to the portal_catalog. See
    http://maurits.vanrees.org/weblog/archive/2009/12/catalog for more
    information.
    """
    def extras(title, index_type='Okapi BM25 Rank',
               lexicon_id='plone_lexicon'):
        # See http://old.zope.org/Members/dedalu/ZCTextIndex_python
        extras = Empty()
        extras.doc_attr = title
        extras.index_type = index_type
        extras.lexicon_id = lexicon_id
        return extras

    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    profile = 'profile-collective.nitf:default'
    setup = api.portal.get_tool('portal_setup')
    setup.runImportStepFromProfile(profile, 'catalog')

    catalog = api.portal.get_tool('portal_catalog')
    indexes = catalog.indexes()

    wanted = (
        ('subtitle', 'ZCTextIndex'),
        ('byline', 'ZCTextIndex'),
        ('genre', 'FieldIndex'),
        ('section', 'FieldIndex'),
        ('urgency', 'FieldIndex'),
        ('location', 'ZCTextIndex'),
        ('SearchableText', 'ZCTextIndex'),
    )

    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            if meta_type == 'ZCTextIndex':
                catalog.addIndex(name, meta_type, extras(name))
            else:
                catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info('Added %s for field %s.', meta_type, name)

    if len(indexables) > 0:
        logger.info('Indexing new indexes %s.', ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def import_various(context):
    """ Import step for configuration that is not handled in XML files.
    """
    # Only run step if a flag file is present
    if context.readDataFile('collective.nitf.marker.txt') is None:
        return

    logger = context.getLogger(PROJECTNAME)
    site = context.getSite()
    add_catalog_indexes(site, logger)



# -*- coding: utf-8 -*-
"""Hacks to work around API inconsistencies between Archetypes and Dexterity."""


def set_image_field(obj, image, content_type):
    """Set image field in object on both, Archetypes and Dexterity."""
    from plone.namedfile.file import NamedBlobImage
    try:
        obj.setImage(image)  # Archetypes
    except AttributeError:
        # Dexterity
        data = image if type(image) == str else image.getvalue()
        obj.image = NamedBlobImage(data=data, contentType=content_type)
    finally:
        obj.reindexObject()


def set_file_field(obj, file, content_type):
    """Set file field in object on both, Archetypes and Dexterity."""
    from plone.namedfile.file import NamedBlobFile
    try:
        obj.setFile(file)  # Archetypes
    except AttributeError:
        # Dexterity
        obj.file = NamedBlobFile(data=file, contentType=content_type)
    finally:
        obj.reindexObject()


def set_text_field(obj, text):
    """Set text field in object on both, Archetypes and Dexterity."""
    from plone.app.textfield.value import RichTextValue
    try:
        obj.setText(text)  # Archetypes
    except AttributeError:
        obj.text = RichTextValue(text, 'text/html', 'text/html')  # Dexterity
    finally:
        obj.reindexObject()



# -*- coding: utf-8 -*-
from collective.nitf.config import DEFAULT_URGENCY
from collective.nitf.config import PROJECTNAME
from collective.nitf.controlpanel import INITFSettings
from collective.nitf.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import logout
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import unittest


class ControlPanelTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.controlpanel = self.portal['portal_controlpanel']

    def test_controlpanel_has_view(self):
        request = self.layer['request']
        view = api.content.get_view(u'nitf-settings', self.portal, request)
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse('@@nitf-settings')

    def test_controlpanel_installed(self):
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertIn('nitf', actions, 'control panel not installed')

    def test_controlpanel_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']

        with api.env.adopt_roles(['Manager']):
            qi.uninstallProducts(products=[PROJECTNAME])

        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertNotIn('nitf', actions, 'control panel not removed')


class RegistryTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(INITFSettings)

    def test_available_sections_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'available_sections'))
        self.assertEqual(self.settings.available_sections, set([u'General']))

    def test_default_section_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'default_section'))
        self.assertEqual(self.settings.default_section, u'General')

    def test_available_genres_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'available_genres'))
        self.assertEqual(self.settings.available_genres, [u'Current'])

    def test_default_genre_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'default_genre'))
        self.assertEqual(self.settings.default_genre, u'Current')

    def test_default_urgency_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'default_urgency'))
        self.assertEqual(self.settings.default_urgency, DEFAULT_URGENCY)

    def test_records_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']

        with api.env.adopt_roles(['Manager']):
            qi.uninstallProducts(products=[PROJECTNAME])

        BASE_REGISTRY = 'collective.nitf.controlpanel.INITFSettings.%s'
        records = [
            BASE_REGISTRY % 'available_sections',
            BASE_REGISTRY % 'default_section',
            BASE_REGISTRY % 'available_genres',
            BASE_REGISTRY % 'default_genre',
            BASE_REGISTRY % 'default_urgency',
            BASE_REGISTRY % 'relatable_content_types',
        ]

        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-

from collective.nitf import _
from collective.nitf import config
from collective.nitf.config import DEFAULT_GENRE
from collective.nitf.config import DEFAULT_SECTION
from plone.app.registry.browser import controlpanel
from plone.directives import form
from zope import schema


PORTALTYPES = 'plone.app.vocabularies.ReallyUserFriendlyTypes'


class INITFSettings(form.Schema):
    """ Interface for the control panel form.
    """
    form.widget(available_sections='z3c.form.browser.textlines.TextLinesFieldWidget')
    available_sections = schema.Set(
        title=_(u'Available Sections'),
        description=_(u'List of available sections in the site.'),
        required=True,
        default=set([DEFAULT_SECTION]),
        value_type=schema.TextLine(title=_(u'Section')),
    )

    default_section = schema.Choice(
        title=_(u'Default Section'),
        description=_(u'Section to be used as default on new items.'),
        required=True,
        vocabulary=u'collective.nitf.AvailableSections',
        default=DEFAULT_SECTION,
    )

    available_genres = schema.List(
        title=_(u'Available Genres'),
        description=_(u'Select the list of available genres in the site. '
                      u'Genres indicate a nature, journalistic or '
                      u'intellectual characteristic of items.'),
        required=True,
        default=[DEFAULT_GENRE],
        value_type=schema.Choice(vocabulary=u'collective.nitf.Genres'),
    )

    default_genre = schema.Choice(
        title=_(u'Default Genre'),
        description=_(u'Genre to be used as default on new items.'),
        required=True,
        vocabulary=u'collective.nitf.Genres',
        default=DEFAULT_GENRE,
    )

    default_urgency = schema.Choice(
        title=_(u'Default Urgency'),
        vocabulary=u'collective.nitf.Urgencies',
        required=True,
        default=config.DEFAULT_URGENCY,
    )


class NITFSettingsEditForm(controlpanel.RegistryEditForm):
    schema = INITFSettings
    label = _(u'NITF Settings')
    description = _(u'Here you can modify the settings for collective.nitf.')


class NITFSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = NITFSettingsEditForm



# -*- coding: utf-8 -*-
from collective.nitf.interfaces import INITF
from Products.CMFPlone.browser.syndication.adapters import DexterityItem
from Products.CMFPlone.interfaces.syndication import IFeed
from zope.component import adapter


@adapter(INITF, IFeed)
class BylineFeed(DexterityItem):

    """Adapter to honor the author of a News Article in the syndication feed."""

    @property
    def author_name(self):
        return self.context.byline



# -*- coding: utf-8 -*-
from collective.nitf.testing import INTEGRATION_TESTING
from collective.nitf.testing import IS_PLONE_5
from plone import api
from plone.registry import field
from plone.registry.interfaces import IRegistry
from plone.registry.record import Record
from zope.component import getUtility

import unittest


class UpgradeTestCaseBase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self, from_version, to_version):
        self.portal = self.layer['portal']
        self.setup = self.portal['portal_setup']
        self.profile_id = u'collective.nitf:default'
        self.from_version = from_version
        self.to_version = to_version

    def get_upgrade_step(self, title):
        """Get the named upgrade step."""
        self.setup.setLastVersionForProfile(self.profile_id, self.from_version)
        upgrades = self.setup.listUpgrades(self.profile_id)
        steps = [s for s in upgrades[0] if s['title'] == title]
        return steps[0] if steps else None

    def execute_upgrade_step(self, step):
        """Execute an upgrade step."""
        request = self.layer['request']
        request.form['profile_id'] = self.profile_id
        request.form['upgrades'] = [step['id']]
        self.setup.manage_doUpgrades(request=request)

    @property
    def total_steps(self):
        """Return the number of steps in the upgrade."""
        self.setup.setLastVersionForProfile(self.profile_id, self.from_version)
        upgrades = self.setup.listUpgrades(self.profile_id)
        assert len(upgrades) > 0
        return len(upgrades[0])


class to1007TestCase(UpgradeTestCaseBase):

    def setUp(self):
        UpgradeTestCaseBase.setUp(self, u'1006', u'1007')

    def test_upgrade_to_2000_registrations(self):
        version = self.setup.getLastVersionForProfile(self.profile_id)[0]
        self.assertTrue(version >= self.to_version)
        self.assertEqual(self.total_steps, 1)

    def test_add_locking_behavior(self):
        title = u'Add locking behavior'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        locking_behavior = 'plone.app.lockingbehavior.behaviors.ILocking'
        types = self.portal['portal_types']
        nitf = types['collective.nitf.content']

        # simulate state on previous version
        behaviors = list(nitf.behaviors)
        behaviors.remove(locking_behavior)
        nitf.behaviors = tuple(behaviors)
        self.assertNotIn(locking_behavior, nitf.behaviors)

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        self.assertIn(locking_behavior, nitf.behaviors)


class to2000TestCase(UpgradeTestCaseBase):

    def setUp(self):
        UpgradeTestCaseBase.setUp(self, u'1008', u'2000')

    def test_upgrade_to_2000_registrations(self):
        version = self.setup.getLastVersionForProfile(self.profile_id)[0]
        self.assertTrue(version >= self.to_version)
        self.assertEqual(self.total_steps, 4)

    @unittest.skipIf(IS_PLONE_5, 'Test not supported under Plone 5')
    def test_character_counter_css_resources(self):
        title = u'Miscellaneous'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        csstool = self.portal['portal_css']
        old_css = '++resource++collective.nitf/charcount.css'
        new_css = '++resource++collective.nitf/styles.css'

        # simulate state on previous version
        csstool.registerResource(old_css)
        self.assertIn(old_css, csstool.getResourceIds())
        csstool.unregisterResource(new_css)
        self.assertNotIn(new_css, csstool.getResourceIds())

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        self.assertIn(new_css, csstool.getResourceIds())

    @unittest.skipIf(IS_PLONE_5, 'Test not supported under Plone 5')
    def test_character_counter_js_resources(self):
        title = u'Miscellaneous'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        jstool = self.portal['portal_javascripts']
        old_js = [
            'characters-count.js',
            '++resource++collective.nitf/nitf_fixes.js',
        ]
        new_js = '++resource++collective.nitf/nitf.js'

        # simulate state on previous version
        for js in old_js:
            jstool.registerResource(js)
            self.assertIn(js, jstool.getResourceIds())
        jstool.unregisterResource(new_js)
        self.assertNotIn(new_js, jstool.getResourceIds())

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        for js in old_js:
            self.assertNotIn(js, jstool.getResourceIds())
        self.assertIn(new_js, jstool.getResourceIds())

    def test_character_counter_registry_records_removed(self):
        title = u'Miscellaneous'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        registry = getUtility(IRegistry)
        BASE_REGISTRY = 'collective.nitf.controlpanel.INITFCharCountSettings.'
        records = [
            BASE_REGISTRY + 'description_max_chars',
            BASE_REGISTRY + 'description_optimal_chars',
            BASE_REGISTRY + 'show_description_counter',
            BASE_REGISTRY + 'show_title_counter',
            BASE_REGISTRY + 'title_max_chars',
            BASE_REGISTRY + 'title_optimal_chars',
        ]

        # simulate state on previous version
        for r in records:
            registry.records[r] = Record(field.TextLine(title=u'Test'))
            self.assertIn(r, registry)

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        for r in records:
            self.assertNotIn(r, registry)

    def test_news_article_registered_views(self):
        title = u'Miscellaneous'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        ttool = self.portal['portal_types']
        fti = ttool['collective.nitf.content']

        # simulate state on previous version
        view_methods = list(fti.view_methods)
        view_methods.remove('slideshow_view')
        view_methods.remove('text_only_view')
        view_methods.append('nitf_galleria')
        fti.view_methods = tuple(view_methods)
        self.assertEqual(len(fti.view_methods), 2)
        self.assertItemsEqual(fti.view_methods, ['view', 'nitf_galleria'])

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        self.assertEqual(len(fti.view_methods), 3)
        self.assertItemsEqual(
            fti.view_methods, ['view', 'slideshow_view', 'text_only_view'])

    def test_update_news_articles_layouts(self):
        title = u'Update News Articles layouts'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        # create a news article and set the layout to 'nitf_galleria'
        with api.env.adopt_roles(['Manager']):
            folder = api.content.create(self.portal, 'Folder', 'folder')
        n1 = api.content.create(folder, 'collective.nitf.content', 'n1')

        n1.setLayout('nitf_galleria')
        self.assertEqual(n1.getLayout(), 'nitf_galleria')

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        self.assertEqual(n1.getLayout(), 'slideshow_view')

    def test_install_new_dependencies(self):
        # check if the upgrade step is registered
        title = u'Install new dependencies'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        # simulate state on previous version
        dependencies = ('collective.js.cycle2',)
        qi = api.portal.get_tool('portal_quickinstaller')
        qi.uninstallProducts(dependencies)
        for p in dependencies:
            self.assertFalse(qi.isProductInstalled(p))

        # execute upgrade step and verify changes were applied
        self.execute_upgrade_step(step)

        for p in dependencies:
            self.assertTrue(qi.isProductInstalled(p))

    def test_update_configlet(self):
        # check if the upgrade step is registered
        title = u'Update control panel configlet'
        step = self.get_upgrade_step(title)
        self.assertIsNotNone(step)

        # simulate state on previous version
        cptool = api.portal.get_tool('portal_controlpanel')
        configlet = cptool.getActionObject('Products/nitf')
        configlet.permissions = old_permissions = ('cmf.ManagePortal',)
        self.assertEqual(configlet.getPermissions(), old_permissions)

        # run the upgrade step to validate the update
        self.execute_upgrade_step(step)
        configlet = cptool.getActionObject('Products/nitf')
        new_permissions = ('collective.nitf: Setup',)
        self.assertEqual(configlet.getPermissions(), new_permissions)



# -*- coding: utf-8 -*-
from collective.cover.tiles.basic import BasicTile
from collective.cover.tiles.basic import IBasicTile
from collective.cover.tiles.configuration_view import IDefaultConfigureForm
from collective.nitf import _
from collective.nitf.interfaces import INITF
from plone.autoform import directives as form
from plone.tiles.interfaces import ITileDataManager
from plone.uuid.interfaces import IUUID
from zope import schema
from zope.browserpage import ViewPageTemplateFile
from zope.interface import implementer


class INITFTile(IBasicTile):

    """A tile that shows information about a News Article."""

    subtitle = schema.Text(
        title=_(u'Subtitle'),
        required=False,
    )

    form.omitted('section')
    form.no_omit(IDefaultConfigureForm, 'section')
    section = schema.Text(
        title=_(u'Section'),
        required=False,
    )

    media_producer = schema.TextLine(
        # nitf/body/body.content/media/media-producer
        title=_(u'Image Rights'),
        required=False,
    )


@implementer(INITFTile)
class NITFTile(BasicTile):

    """A tile that shows information about a News Article."""

    index = ViewPageTemplateFile('nitf.pt')
    is_configurable = True
    is_editable = True
    is_droppable = True

    short_name = _(u'msg_short_name_nitf', u'News Article')

    def accepted_ct(self):
        """Return a list of content types accepted by the tile."""
        return ['collective.nitf.content']

    def populate_with_object(self, obj):
        super(BasicTile, self).populate_with_object(obj)

        if INITF.providedBy(obj):
            image = obj.image()
            data = dict(
                title=obj.title,
                description=obj.description,
                subtitle=obj.subtitle,
                section=obj.section,
                uuid=IUUID(obj),
                date=True,
                subjects=True,
                image=self.get_image_data(image),
                media_producer=obj.media_producer,
            )
            # clear scales as new image is getting saved
            self.clear_scales()

            data_mgr = ITileDataManager(self)
            data_mgr.set(data)

    def _get_field_configuration(self, field):
        """Return a dict with the configuration of the field. This is a
        helper function to deal with the ugliness of the internal data
        structure.
        """
        fields = self.get_configured_fields()
        return [f for f in fields if f['id'] == field][0]

    @property
    def title_tag(self):
        field = self._get_field_configuration('title')
        tag, title, href = field['htmltag'], field['content'], self.getURL()
        return u"""
            <{tag}>
              <a href="{href}">{title}</a>
            </{tag}>
            """.format(tag=tag, title=title, href=href)



# -*- coding: utf-8 -*-
from collective.nitf import config as c
from collective.nitf.testing import INTEGRATION_TESTING
from plone import api

import unittest


class CollectionTypeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        with api.env.adopt_roles(['Manager']):
            self.folder = api.content.create(
                self.portal, 'Folder', 'test')
            self.c1 = api.content.create(
                self.portal, 'Collection', 'c1')

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            }
        ])

    def test_urgency_filter(self, ):
        self.assertEqual(len(self.c1.queryCatalog()), 0)

        n1 = api.content.create(self.folder, 'collective.nitf.content', 'n1')
        n1.urgency = c.LOW
        n1.reindexObject()
        self.assertEqual(len(self.c1.queryCatalog()), 1)

        n2 = api.content.create(self.folder, 'collective.nitf.content', 'n2')
        n2.urgency = c.NORMAL
        n2.reindexObject()
        self.assertEqual(len(self.c1.queryCatalog()), 2)

        n3 = api.content.create(self.folder, 'collective.nitf.content', 'n3')
        n3 = self.folder['n3']
        n3.urgency = c.HIGH
        n3.reindexObject()
        self.assertEqual(len(self.c1.queryCatalog()), 3)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.LOW]
            }
        ])
        self.assertEqual(len(self.c1.queryCatalog()), 1)
        expected = self.c1.queryCatalog()[0].getObject()
        self.assertEqual(n1, expected)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.NORMAL]
            }
        ])
        self.assertEqual(len(self.c1.queryCatalog()), 1)
        expected = self.c1.queryCatalog()[0].getObject()
        self.assertEqual(n2, expected)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.HIGH]
            }
        ])
        self.assertEqual(len(self.c1.queryCatalog()), 1)
        expected = self.c1.queryCatalog()[0].getObject()
        self.assertEqual(n3, expected)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.NORMAL, c.HIGH]
            }
        ])
        self.assertEqual(len(self.c1.queryCatalog()), 2)
        expected = [b.getObject() for b in self.c1.queryCatalog()]
        self.assertEqual([n2, n3], expected)



# -*- coding: utf-8 -*-
"""Setup testing infrastructure.

For Plone 5 we need to install plone.app.contenttypes.

Tile for collective.cover is only tested in Plone 4.3.
"""
from plone import api
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import os
import pkg_resources
import shutil

try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    from plone.app.testing import PLONE_FIXTURE
    DEXTERITY_ONLY = False
else:
    from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE as PLONE_FIXTURE
    DEXTERITY_ONLY = True

try:
    pkg_resources.get_distribution('collective.cover')
except pkg_resources.DistributionNotFound:
    HAS_COVER = False
else:
    HAS_COVER = True

IS_PLONE_5 = api.env.plone_version().startswith('5')

# set of images to be used on tests
IMAGES = (
    '640px-Mandel_zoom_00_mandelbrot_set.jpg',
    '640px-Mandel_zoom_04_seehorse_tail.jpg',
    '640px-Mandel_zoom_06_double_hook.jpg',
)


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        if HAS_COVER:
            import collective.cover
            self.loadZCML(package=collective.cover)

        import collective.nitf
        self.loadZCML(package=collective.nitf)

    def setUpPloneSite(self, portal):
        if HAS_COVER:
            self.applyProfile(portal, 'collective.cover:default')

        self.applyProfile(portal, 'collective.nitf:default')

        portal_workflow = portal['portal_workflow']
        portal_workflow.setChainForPortalTypes(
            ('collective.nitf.content',), 'simple_publication_workflow')

        for i in IMAGES:
            origin = os.path.join(os.path.dirname(__file__), 'tests', i)
            shutil.copy2(origin, '/tmp')


def get_image(name):
    image = os.path.join(os.path.dirname(__file__), 'tests', name)
    return open(image).read()


def generate_text(size):
    import random
    import string
    chars = string.letters + string.digits
    return ''.join(random.choice(chars) for x in range(size))


# TODO: simplify this using a testfixture profile
class RobotFixture(Fixture):

    def setUpPloneSite(self, portal):
        super(RobotFixture, self).setUpPloneSite(portal)
        with api.env.adopt_roles(['Manager']):
            api.content.create(portal, 'collective.nitf.content', 'related')
            obj = api.content.create(portal, 'collective.nitf.content', 'n1')

        from collective.nitf.tests.api_hacks import set_image_field
        api.content.create(obj, 'Image', 'img1')
        api.content.create(obj, 'Image', 'img2')
        api.content.create(obj, 'Image', 'img3')
        set_image_field(obj['img1'], get_image(IMAGES[0]), 'image/jpeg')
        set_image_field(obj['img2'], get_image(IMAGES[1]), 'image/jpeg')
        set_image_field(obj['img3'], get_image(IMAGES[2]), 'image/jpeg')
        intids = getUtility(IIntIds)
        to_id = intids.getId(portal.related)
        portal.n1.relatedItems = [RelationValue(to_id)]
        open('/tmp/random.txt', 'w').write(generate_text(256))


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name='collective.nitf:Integration')

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name='collective.nitf:Functional')

ROBOT_FIXTURE = RobotFixture()
ROBOT_TESTING = FunctionalTesting(
    bases=(ROBOT_FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, z2.ZSERVER_FIXTURE),
    name='collective.nitf:Robot',
)

# first image as a string
FRACTAL = get_image(IMAGES[0])



# -*- coding: utf-8 -*-
from collective.nitf.interfaces import INITF
from collective.nitf.testing import INTEGRATION_TESTING
from collective.nitf.tests.api_hacks import set_image_field
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest


zptlogo = (
    'GIF89a\x10\x00\x10\x00\xd5\x00\x00\xff\xff\xff\xff\xff\xfe\xfc\xfd\xfd'
    '\xfa\xfb\xfc\xf7\xf9\xfa\xf5\xf8\xf9\xf3\xf6\xf8\xf2\xf5\xf7\xf0\xf4\xf6'
    '\xeb\xf1\xf3\xe5\xed\xef\xde\xe8\xeb\xdc\xe6\xea\xd9\xe4\xe8\xd7\xe2\xe6'
    '\xd2\xdf\xe3\xd0\xdd\xe3\xcd\xdc\xe1\xcb\xda\xdf\xc9\xd9\xdf\xc8\xd8\xdd'
    '\xc6\xd7\xdc\xc4\xd6\xdc\xc3\xd4\xda\xc2\xd3\xd9\xc1\xd3\xd9\xc0\xd2\xd9'
    '\xbd\xd1\xd8\xbd\xd0\xd7\xbc\xcf\xd7\xbb\xcf\xd6\xbb\xce\xd5\xb9\xcd\xd4'
    '\xb6\xcc\xd4\xb6\xcb\xd3\xb5\xcb\xd2\xb4\xca\xd1\xb2\xc8\xd0\xb1\xc7\xd0'
    '\xb0\xc7\xcf\xaf\xc6\xce\xae\xc4\xce\xad\xc4\xcd\xab\xc3\xcc\xa9\xc2\xcb'
    '\xa8\xc1\xca\xa6\xc0\xc9\xa4\xbe\xc8\xa2\xbd\xc7\xa0\xbb\xc5\x9e\xba\xc4'
    '\x9b\xbf\xcc\x98\xb6\xc1\x8d\xae\xbaFgs\x00\x00\x00\x00\x00\x00\x00\x00'
    '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    '\x00,\x00\x00\x00\x00\x10\x00\x10\x00\x00\x06z@\x80pH,\x12k\xc8$\xd2f\x04'
    '\xd4\x84\x01\x01\xe1\xf0d\x16\x9f\x80A\x01\x91\xc0ZmL\xb0\xcd\x00V\xd4'
    '\xc4a\x87z\xed\xb0-\x1a\xb3\xb8\x95\xbdf8\x1e\x11\xca,MoC$\x15\x18{'
    '\x006}m\x13\x16\x1a\x1f\x83\x85}6\x17\x1b $\x83\x00\x86\x19\x1d!%)\x8c'
    '\x866#\'+.\x8ca`\x1c`(,/1\x94B5\x19\x1e"&*-024\xacNq\xba\xbb\xb8h\xbeb'
    '\x00A\x00;')


class ContentTypeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        with api.env.adopt_roles(['Manager']):
            self.n1 = api.content.create(
                self.portal, 'collective.nitf.content', 'n1')

    def test_adding(self):
        self.assertTrue(INITF.providedBy(self.n1))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        self.assertIsNotNone(fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        schema = fti.lookupSchema()
        self.assertEqual(INITF, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(INITF.providedBy(new_object))

    def test_default_genre(self):
        from collective.nitf.config import DEFAULT_GENRE
        self.assertEqual(self.n1.genre, DEFAULT_GENRE)

    def test_default_urgency(self):
        from collective.nitf.config import DEFAULT_URGENCY
        self.assertEqual(self.n1.urgency, DEFAULT_URGENCY)

    def test_section_behavior(self):
        from collective.nitf.behaviors.interfaces import ISection
        self.assertTrue(ISection.providedBy(self.n1))

    def test_is_selectable_as_folder_default_view(self):
        self.portal.setDefaultPage('n1')
        self.assertEqual(self.portal.default_page, 'n1')

    def test_action_is_registered(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        actions = [a.id for a in fti.listActions()]
        self.assertIn('media', actions)

    def test_is_empty(self):
        # the new article has no content inside
        self.assertTrue(self.n1.is_empty())

        # Image doesn't count, so it should be still empty
        self.n1.invokeFactory('Image', 'foo')
        self.assertTrue(self.n1.is_empty())

        # File and Link count
        self.n1.invokeFactory('File', 'bar')
        self.assertFalse(self.n1.is_empty())
        self.n1.manage_delObjects('bar')

        self.n1.invokeFactory('Link', 'baz')
        self.assertFalse(self.n1.is_empty())

    def test_image_scale(self):
        self.assertIsNone(self.n1.image())

        img = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(img, zptlogo, 'image/gif')

        scales = self.n1.unrestrictedTraverse('@@images')
        scale = scales.scale('image', 'thumb')
        self.assertEqual(scale.height, 16)
        self.assertEqual(scale.width, 16)

    def test_media_caption(self):
        self.assertEqual(self.n1.media_caption(), u'')

        img = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(img, zptlogo, 'image/gif')
        self.assertEqual(self.n1.media_caption(), 'bar')

    def test_media_producer(self):
        self.assertEqual(self.n1.media_producer(), u'')

        img = api.content.create(self.n1, 'Image', title='foo', rights='bar')
        set_image_field(img, zptlogo, 'image/gif')
        self.assertEqual(self.n1.media_producer(), 'bar')

    def test_image_thumb(self):
        self.assertIsNone(self.n1.image_thumb())

        img = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(img, zptlogo, 'image/gif')
        self.assertIsNotNone(self.n1.image_thumb())

    def test_tag(self):
        self.assertIsNone(self.n1.tag())

        img = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(img, zptlogo, 'image/gif')

        # image alt and title attributes
        self.assertIn('alt="foo" title="foo"', self.n1.tag())

        # image size
        self.assertIn('height="16" width="16', self.n1.tag())

        # image scale using @@images
        self.assertIn(
            'src="http://nohost/plone/n1/foo/@@images/', self.n1.tag(scale='preview'))

        # image class
        self.assertIn('class="myClass"', self.n1.tag(css_class='myClass'))



# -*- coding: utf-8 -*-
from collective.nitf.config import JS_RESOURCES
from plone import api
from plone.app.layout.viewlets.content import DocumentBylineViewlet
from plone.dexterity.browser.view import DefaultView
from plone.memoize import ram
from time import time


class View(DefaultView):

    """Default view of a News Article."""

    def _get_brains(self, object_name=None):
        """Return a list of brains inside the NITF object."""
        catalog = api.portal.get_tool('portal_catalog')
        path = '/'.join(self.context.getPhysicalPath())
        brains = catalog(Type=object_name, path=path,
                         sort_on='getObjPositionInParent')

        return brains

    def get_images(self):
        """Return a list of image brains inside the NITF object."""
        return self._get_brains('Image')

    def get_files(self):
        """Return a list of file brains inside the NITF object."""
        return self._get_brains('File')

    def get_links(self):
        """Return a list of link brains inside the NITF object."""
        return self._get_brains('Link')

    def get_media(self):
        """Return a list of object brains inside the NITF object."""
        media_ct = [x.title for x in self.context.allowedContentTypes()]
        return self._get_brains(media_ct)


class Slideshow(DefaultView):

    """Slideshow view of a News Article."""

    def id(self):
        return id(self)

    def js_resources(self):
        """Return a list of JS resources that are not available in the
        registry, but need to be loaded anyway. This way the slideshow
        could use resources registered locally or globally.

        :returns: list of ids
        :rtype: list
        """
        js_registry = api.portal.get_tool('portal_javascripts')
        global_resources = js_registry.getResourceIds()
        return [r for r in JS_RESOURCES if r not in global_resources]


class TextOnly(DefaultView):

    """Text only view of a News Article."""


class NITFBylineViewlet(DocumentBylineViewlet):

    """Override the document byline viewlet to include semantic markup."""

    def _search_member_by_name(self, fullname):
        """Search a user by its full name and return its member
        information.

        :param fullname: full name of the user we are looking for
        :type fullname: unicode
        :returns: member information
        :rtype: dict
        """
        if not fullname:
            return None

        membership = api.portal.get_tool('portal_membership')
        members = membership.searchForMembers(name=fullname)
        if members:
            # in case there are more than one members with the
            # same fullname, we use the first one listed
            member = members[0].getUserId()
            return membership.getMemberInfo(member)

    @ram.cache(lambda method, self, fullname: (time() // 60, fullname))
    def search_member_by_name(self, fullname):
        """Cached version of _search_member_by_name. Caching is done
        for one minute to avoid performance issues when having many
        users on sites using LDAP authentication.
        """
        return self._search_member_by_name(fullname)

    @property
    def author_id(self):
        member = self.search_member_by_name(self.context.byline)
        if member:
            return member['username']

    def author(self):
        return self.search_member_by_name(self.context.byline)

    def authorname(self):
        return self.context.byline



# -*- coding:utf-8 -*-
from collective.nitf.config import PROJECTNAME
from plone.app.upgrade.utils import loadMigrationProfile

import logging


def fix_collections(context):
    """Update to 1008 version."""
    logger = logging.getLogger(PROJECTNAME)
    collections = context.portal_catalog(portal_type='Collection')
    logger.info('About to update {0} objects'.format(len(collections)))
    for col in collections:
        obj = col.getObject()

        # XXX: To fix the query we need a list of dicts describing it.
        #      With collections based on Archetypes we use getRawQuery;
        #      with collections based on Dexterity (e.g., if
        #      plone.app.contenttypes is installed) we need to use
        #      getQuery because we have an API inconsistency.
        #      See: https://github.com/plone/plone.app.contenttypes/issues/283
        try:
            query = obj.getRawQuery()  # Archetypes
        except AttributeError:
            query = obj.getQuery()  # Dexterity

        fixed_query = []
        for item in query:
            fixed_item = dict(item)
            if item['i'] == 'urgency':
                fixed_item['o'] = 'plone.app.querystring.operation.intselection.is'
            fixed_query.append(fixed_item)
        obj.setQuery(fixed_query)
        logger.info(
            'Collection {0} at {1} updated'.format(col.id, col.getPath())
        )
    logger.info('Done')


def apply_profile(context):
    """Update to 1008 version."""
    logger = logging.getLogger(PROJECTNAME)
    profile = 'profile-collective.nitf.upgrades.v1008:default'
    loadMigrationProfile(context, profile)
    logger.info('Updated to version 1008')



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '2.0b2'
description = "A content type inspired on the IPTC's News Industry Text Format specification."
long_description = (
    open('README.rst').read() + '\n' +
    open('CONTRIBUTORS.rst').read() + '\n' +
    open('CHANGES.rst').read()
)

setup(name='collective.nitf',
      version=version,
      description=description,
      long_description=long_description,
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Operating System :: OS Independent',
          'Programming Language :: JavaScript',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Office/Business :: News/Diary',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],
      keywords='plone dexterity iptc newsml nitf',
      author='Héctor Velarde',
      author_email='hector.velarde@gmail.com',
      url='https://github.com/collective/collective.nitf',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'AccessControl',
          'Acquisition',
          'collective.js.cycle2 >=1.0b1',
          'collective.js.jqueryui',
          'collective.prettydate >=1.1',
          'plone.api',
          'plone.app.contentmenu',
          'plone.app.dexterity [relations]',
          'plone.app.imaging',
          'plone.app.layout >=2.3.12',
          'plone.app.lockingbehavior',
          'plone.app.portlets',
          'plone.app.querystring >=1.2.5',
          'plone.app.registry',
          'plone.app.relationfield',
          'plone.app.textfield',
          'plone.app.upgrade',
          'plone.app.vocabularies',
          'plone.behavior',
          'plone.dexterity',
          'plone.directives.form',
          'plone.formwidget.autocomplete',
          'plone.formwidget.contenttree',
          'plone.indexer',
          'plone.portlets',
          'plone.registry',
          'plone.supermodel',
          'plone.uuid',
          'Products.CMFPlone >=4.3',
          'Products.GenericSetup',
          'setuptools',
          'zope.browserpage',
          'zope.component',
          'zope.deprecation',
          'zope.formlib',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.schema',
      ],
      extras_require={
          'test': [
              'mock',
              'plone.app.customerize',
              'plone.app.robotframework',
              'plone.app.testing [robot]',
              'plone.browserlayer',
              'plone.namedfile',
              'plone.testing',
              'robotframework-selenium2accessibility',
              'robotsuite',
              'z3c.relationfield',
              'zope.intid',
          ],
      },
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-
from collective.nitf.config import JS_RESOURCES
from collective.nitf.interfaces import INITFLayer
from collective.nitf.testing import FRACTAL
from collective.nitf.testing import INTEGRATION_TESTING
from collective.nitf.tests.api_hacks import set_file_field
from collective.nitf.tests.api_hacks import set_image_field
from plone import api
from plone.app.customerize import registration
from plone.app.testing import logout
from zope.interface import alsoProvides

import unittest


class TestViewMixin:

    def test_view_is_registered(self):
        registered = [v.name for v in registration.getViews(INITFLayer)]
        self.assertIn(self.name, registered)


class BaseViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        alsoProvides(self.request, INITFLayer)

        with api.env.adopt_roles(['Manager']):
            self.n1 = api.content.create(
                self.portal, 'collective.nitf.content', 'n1')


class DefaultViewTestCase(TestViewMixin, BaseViewTestCase):

    def setUp(self):
        super(DefaultViewTestCase, self).setUp()
        self.name = u'view'
        self.view = api.content.get_view(self.name, self.n1, self.request)

    def test_is_default_view(self):
        types = self.portal['portal_types']
        default_view = types['collective.nitf.content'].default_view
        self.assertEqual(default_view, u'view')

    # FIXME: we're getting: error while rendering collective.nitf.image
    def test_default_view_render(self):
        self.assertNotIn('<div id="media">', self.view())
        image = api.content.create(self.n1, 'Image', title='foo')
        set_image_field(image, FRACTAL, 'image/jpeg')
        self.assertIn('<div id="media">', self.view())

    def test_render_js_resources(self):
        rendered = self.view()
        for js in JS_RESOURCES:
            self.assertNotIn(js, rendered)

    def test_get_images(self):
        images = self.view.get_images()
        self.assertEqual(len(images), 0)

        image = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(image, FRACTAL, 'image/jpeg')
        images = self.view.get_images()
        self.assertEqual(len(images), 1)
        self.assertEqual(images[0].getObject().Title(), 'foo')
        self.assertEqual(images[0].getObject().Description(), 'bar')

    def test_get_files(self):
        files = self.view.get_files()
        self.assertEqual(len(files), 0)

        file = api.content.create(self.n1, 'File', title='foo', description='bar')
        set_file_field(file, FRACTAL, 'image/jpeg')
        files = self.view.get_files()
        self.assertEqual(len(files), 1)
        self.assertEqual(files[0].getObject().Title(), 'foo')
        self.assertEqual(files[0].getObject().Description(), 'bar')

    def test_get_links(self):
        links = self.view.get_links()
        self.assertEqual(len(links), 0)

        self.n1.invokeFactory('Link', 'foo', remoteUrl='http://foo/')
        links = self.view.get_links()
        self.assertEqual(len(links), 1)
        self.assertEqual(links[0].getObject().id, 'foo')
        self.assertEqual(links[0].getObject().remoteUrl, 'http://foo/')

    def test_get_media(self):
        media = self.view.get_media()
        self.assertEqual(len(media), 0)

        image = api.content.create(self.n1, 'Image', title='foo')
        set_image_field(image, FRACTAL, 'image/jpeg')
        file = api.content.create(self.n1, 'File', title='bar')
        set_file_field(file, FRACTAL, 'image/jpeg')
        self.n1.invokeFactory('Link', 'baz', remoteUrl='http://baz/')
        media = self.view.get_media()
        self.assertEqual(len(media), 3)
        self.assertEqual(media[0].getObject().Title(), 'foo')
        self.assertEqual(media[1].getObject().id, 'bar')
        self.assertEqual(media[2].getObject().id, 'baz')


class SlideshowViewTestCase(TestViewMixin, BaseViewTestCase):

    def setUp(self):
        super(SlideshowViewTestCase, self).setUp()
        self.name = u'slideshow_view'
        self.view = api.content.get_view(self.name, self.n1, self.request)

    def test_slideshow_view_render(self):
        self.assertNotIn('<div class="slideshow-player">', self.view())
        self.assertNotIn('<div class="slideshow-pager">', self.view())
        image = api.content.create(self.n1, 'Image', 'foo')
        set_image_field(image, FRACTAL, 'image/jpeg')
        self.assertIn('<div class="slideshow-player">', self.view())
        self.assertIn('<div class="slideshow-pager">', self.view())

    def test_render_js_resources(self):
        rendered = self.view()
        for js in JS_RESOURCES:
            self.assertIn(js, rendered)


class TextOnlyViewTestCase(TestViewMixin, BaseViewTestCase):

    def setUp(self):
        super(TextOnlyViewTestCase, self).setUp()
        self.name = u'text_only_view'
        self.view = api.content.get_view(self.name, self.n1, self.request)

    def test_text_only_view_render(self):
        self.assertNotIn('<div id="media">', self.view())
        image = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(image, FRACTAL, 'image/jpeg')
        self.assertNotIn('<div id="media">', self.view())

    def test_render_js_resources(self):
        rendered = self.view()
        for js in JS_RESOURCES:
            self.assertNotIn(js, rendered)


class NITFViewTestCase(BaseViewTestCase):

    def setUp(self):
        super(NITFViewTestCase, self).setUp()
        self.view = api.content.get_view(u'nitf', self.n1, self.request)

    def test_get_mediatype(self):
        _get_mediatype = self.view._get_mediatype
        self.assertEqual(_get_mediatype('application/pdf'), 'application')
        self.assertEqual(_get_mediatype('audio/mpeg3'), 'audio')
        self.assertEqual(_get_mediatype('image/jpeg'), 'image')
        self.assertEqual(_get_mediatype('image/png'), 'image')
        self.assertEqual(_get_mediatype('multipart/signed'), 'other')
        self.assertEqual(_get_mediatype('text/plain'), 'text')
        self.assertEqual(_get_mediatype('video/avi'), 'video')

    def test_get_media(self):
        get_media = self.view.get_media
        self.assertEqual(len(get_media()), 0)

        image = api.content.create(self.n1, 'Image', title='foo')
        set_image_field(image, FRACTAL, 'image/jpeg')
        self.assertEqual(len(get_media()), 1)


class NewsMLViewTestCase(BaseViewTestCase):

    def setUp(self):
        super(NewsMLViewTestCase, self).setUp()
        self.view = api.content.get_view(u'newsml', self.n1, self.request)

    def test_version(self):
        self.assertEqual(self.view.version(), 1)

    def test_nitf_size(self):
        self.assertEqual(self.view.nitf_size(), 1000)


class MediaViewTestCase(BaseViewTestCase):

    def test_media_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        with self.assertRaises(Unauthorized):
            self.n1.restrictedTraverse('@@media')


class TraversalViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        with api.env.adopt_roles(['Manager']):
            self.n1 = api.content.create(
                self.portal, 'collective.nitf.content', 'n1')

    def test_images_traversal(self):
        from collective.nitf.testing import DEXTERITY_ONLY
        from collective.nitf.testing import IS_PLONE_5
        image = api.content.create(self.n1, 'Image', title='foo', description='bar')
        set_image_field(image, FRACTAL, 'image/jpeg')
        scale = self.n1.unrestrictedTraverse('@@images').scale('image')

        # FIXME: acquisition?
        if IS_PLONE_5 or DEXTERITY_ONLY:
            # (Pdb) scale
            # <plone.namedfile.scaling.ImageScale object at 0x7eff7b683650>
            self.assertEqual(scale.index_html(), FRACTAL)
        else:
            # (Pdb) scale
            # <plone.app.blob.field.BlobWrapper object at 0x7f85bdb6b8c0>
            self.assertEqual(scale.data, FRACTAL)



# -*- coding: utf-8 -*-

from collective.nitf.controlpanel import INITFSettings
from collective.nitf.testing import INTEGRATION_TESTING
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.component import queryUtility
from zope.schema.interfaces import IVocabularyFactory

import unittest


class VocabulariesTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(INITFSettings)

    def test_genres_vocabulary(self):
        name = 'collective.nitf.Genres'
        util = queryUtility(IVocabularyFactory, name)
        self.assertIsNotNone(util, None)
        genres = util(self.portal)
        self.assertEqual(len(genres), 44)

    def test_available_genres_vocabulary(self):
        name = 'collective.nitf.AvailableGenres'
        util = queryUtility(IVocabularyFactory, name)
        self.assertIsNotNone(util, None)
        available_genres = util(self.portal)
        self.assertEqual(len(available_genres), 1)
        self.assertIn(u'Current', available_genres)

    def test_available_genres_vocabulary_is_sorted(self):
        self.settings.available_genres = [u'Voicer', u'Current', u'Actuality']
        name = 'collective.nitf.AvailableGenres'
        util = getUtility(IVocabularyFactory, name)
        genres = util(self.portal)
        genres = [i.title for i in genres]
        self.assertListEqual(genres, [u'Actuality', u'Current', u'Voicer'])

    def test_available_sections_vocabulary(self):
        name = 'collective.nitf.AvailableSections'
        util = queryUtility(IVocabularyFactory, name)
        self.assertIsNotNone(util, None)
        sections = util(self.portal)
        self.assertEqual(len(sections), 1)
        self.assertIn(u'General', sections)

    def test_available_sections_vocabulary_is_sorted(self):
        self.settings.available_sections = set([u'5', u'4', u'3', u'2', u'1'])
        name = 'collective.nitf.AvailableSections'
        util = getUtility(IVocabularyFactory, name)
        sections = util(self.portal)
        sections = [i.title for i in sections]
        self.assertListEqual(sections, [u'1', u'2', u'3', u'4', u'5'])

    def test_urgencies_vocabulary(self):
        name = 'collective.nitf.Urgencies'
        util = queryUtility(IVocabularyFactory, name)
        self.assertIsNotNone(util, None)
        urgencies = util(self.portal)
        self.assertEqual(len(urgencies), 3)



# -*- coding:utf-8 -*-
from collective.nitf.config import PROJECTNAME
from plone import api

import logging


logger = logging.getLogger(PROJECTNAME)


def add_locking_behaviour(context):
    """Add locking behaviour to News Article."""
    locking_behavior = 'plone.app.lockingbehavior.behaviors.ILocking'
    types = api.portal.get_tool('portal_types')
    nitf = types['collective.nitf.content']
    behaviors = list(nitf.behaviors)
    behaviors.append(locking_behavior)
    nitf.behaviors = tuple(behaviors)
    logger.info('ILocking behavior added to News Article content type.')



# -*- coding: utf-8 -*-
from collective.nitf.testing import DEXTERITY_ONLY
from collective.nitf.testing import IS_PLONE_5
from collective.nitf.testing import ROBOT_TESTING
from plone.testing import layered

import os
import robotsuite
import unittest


files = os.listdir(os.path.dirname(__file__))
tests = [f for f in files if f.startswith('test_') and f.endswith('.robot')]

noncritical = ['Expected Failure']
if DEXTERITY_ONLY:
    # FIXME: https://github.com/collective/collective.nitf/issues/172
    noncritical.append('issue_172')

# skip RobotFramework tests in Plone 5
if IS_PLONE_5:
    tests = []


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(
            robotsuite.RobotTestSuite(t, noncritical=noncritical),
            layer=ROBOT_TESTING)
        for t in tests
    ])
    return suite



# -*- coding: utf-8 -*-
from collective.nitf import _
from collective.nitf.config import GENRES
from collective.nitf.config import URGENCIES
from collective.nitf.controlpanel import INITFSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.schema.vocabulary import SimpleVocabulary

import unicodedata


# TODO: use plone.i18n normalizer
def _normalize_token(token):
    """Normalize a token using ascii as encoding."""
    normalize = unicodedata.normalize
    return normalize('NFKD', token).encode('ascii', 'ignore').lower()


def AvailableGenresVocabulary(context):
    """ Creates a vocabulary with the available genres stored in the registry;
    the vocabulary is normalized to allow the use of non-ASCII characters.
    """
    registry = getUtility(IRegistry)
    settings = registry.forInterface(INITFSettings)
    available_genres = list(settings.available_genres)
    # sort by translated genre
    available_genres.sort(lambda a, b: cmp(_(a), _(b)))
    items = []
    for genre in available_genres:
        token = _normalize_token(genre)
        items.append(SimpleVocabulary.createTerm(genre, token, _(genre)))
    return SimpleVocabulary(items)


def SectionsVocabulary(context):
    """ Creates a vocabulary with the available sections stored in the
    registry; the vocabulary is normalized to allow the use of non-ASCII
    characters.
    """
    registry = getUtility(IRegistry)
    settings = registry.forInterface(INITFSettings)
    available_sections = list(settings.available_sections)
    available_sections.sort()
    items = []
    for section in available_sections:
        token = _normalize_token(section)
        items.append(SimpleVocabulary.createTerm(section, token, section))
    return SimpleVocabulary(items)


def GenresVocabulary(context):
    """ Creates a vocabulary to expose Genres
    """
    return GENRES


def UrgenciesVocabulary(context):
    """ Creates a vocabulary to expose Urgencies
    """
    return URGENCIES



# -*- coding: utf-8 -*-
from plone.app.imaging.scaling import ImageScaling as BaseImageScaling
from Products.CMFPlone.utils import safe_hasattr
from zope.traversing.interfaces import TraversalError


class ImageScaling(BaseImageScaling):

    """Adapter to deal with issues created by our fake image field."""

    def traverse(self, name, furtherPath):
        """Fix image generation using traversal. The only valid option
        for image field name is "image"; that's the difference with
        the original code. We need to find out later if this is a bug
        in plone.app.imaging or not.

        See: https://github.com/collective/collective.nitf/pull/171
        """
        if not furtherPath:
            if safe_hasattr(self, '_image_fieldname'):
                scale_name = name
                name = self._image_fieldname
            else:
                scale_name = None
            image = self.scale(name, scale_name)
            if image is not None:
                return image.tag()
            raise TraversalError(self, name)
        if name == 'image':
            self._image_fieldname = name
            return self
        raise TraversalError(self, name)

    def scale(self, fieldname=None, scale=None, height=None, width=None, **kwargs):
        """Fix image generation on summary_view in collections."""
        if fieldname == 'image':
            image = self.context.image()
            scales = image.restrictedTraverse('@@images')
            return scales.scale(fieldname, scale, height, width, **kwargs)
        else:
            # XXX: check if we're getting here at some point
            return super(ImageScaling, self).scale(
                fieldname, scale, height, width, **kwargs)



# -*- coding: utf-8 -*-
from collective.nitf.browser import View


class NewsML(View):

    """Shows News Article in NewsML format."""

    def version(self):
        """Returns news article revision number."""
        # TODO: get revision number
        return 1

    def nitf_size(self):
        """Returns size of the News Article."""
        # TODO: calculate size
        return 1000

    ITEM_REF = """
<itemRef href="%s/@@nitf" size="%s"
   contenttype="application/nitf+xml" format="fmt:nitf">
    <title>%s</title>
</itemRef>"""

    def get_related_items(self):
        """Returns an itemRef tag for each related item (only News Articles).
        """
        items = getattr(self.context, 'relatedItems', None)
        if items is not None:
            related_items = []
            for i in items:
                href = i.to_object.absolute_url()
                size = self.nitf_size()
                title = i.to_object.Title()
                item_ref = self.ITEM_REF % (href, size, title)
                related_items.append(item_ref)
            return related_items



# -*- coding: utf-8 -*-
from collective.nitf.config import PROJECTNAME
from collective.nitf.testing import DEXTERITY_ONLY
from collective.nitf.testing import HAS_COVER
from collective.nitf.testing import INTEGRATION_TESTING
from collective.nitf.testing import IS_PLONE_5
from plone import api
from plone.browserlayer.utils import registered_layers

import unittest


DEPENDENCIES = (
    'collective.js.cycle2',
    'collective.js.jqueryui',
)

JS = '++resource++collective.nitf/nitf.js'
CSS = '++resource++collective.nitf/styles.css'


class InstallTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal['portal_quickinstaller']

    def test_installed(self):
        self.assertTrue(self.qi.isProductInstalled(PROJECTNAME))

    def test_dependencies_installed(self):
        for p in DEPENDENCIES:
            self.assertTrue(
                self.qi.isProductInstalled(p), '{0} not installed'.format(p))

    def test_setup_permission(self):
        permission = 'collective.nitf: Setup'
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        expected = ['Manager', 'Site Administrator']
        self.assertListEqual(roles, expected)

    def test_add_permission(self):
        permission = 'collective.nitf: Add News Article'
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        expected = ['Contributor', 'Manager', 'Owner', 'Site Administrator']
        self.assertListEqual(roles, expected)

    def test_addon_layer(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertIn('INITFLayer', layers)

    def test_link_workflow_changed(self):
        workflow_tool = self.portal.portal_workflow
        chain = workflow_tool.getChainForPortalType('Link')
        self.assertEqual(len(chain), 1)
        self.assertEqual(chain[0], 'one_state_workflow')

    @unittest.skipIf(IS_PLONE_5, 'No easy way to test this under Plone 5')
    def test_jsregistry(self):
        resource_ids = self.portal.portal_javascripts.getResourceIds()
        self.assertIn(JS, resource_ids)

    @unittest.skipIf(IS_PLONE_5, 'No easy way to test this under Plone 5')
    def test_cssregistry(self):
        resource_ids = self.portal.portal_css.getResourceIds()
        self.assertIn(CSS, resource_ids)

    @unittest.skipUnless(HAS_COVER, 'plone.app.tiles must be installed')
    def test_tile(self):
        tiles = api.portal.get_registry_record('plone.app.tiles')
        self.assertIn(u'collective.nitf', tiles)


class UninstallTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal['portal_quickinstaller']

        with api.env.adopt_roles(['Manager']):
            self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))

    def test_addon_layer_removed(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertNotIn('INITFLayer', layers)

    @unittest.skipIf(IS_PLONE_5, 'No easy way to test this under Plone 5')
    def test_jsregistry_removed(self):
        resource_ids = self.portal.portal_javascripts.getResourceIds()
        self.assertNotIn(JS, resource_ids)

    @unittest.skipIf(IS_PLONE_5, 'No easy way to test this under Plone 5')
    def test_cssregistry_removed(self):
        resource_ids = self.portal.portal_css.getResourceIds()
        self.assertNotIn(CSS, resource_ids)

    @unittest.skipUnless(HAS_COVER, 'plone.app.tiles must be installed')
    def test_tile_removed(self):
        tiles = api.portal.get_registry_record('plone.app.tiles')
        self.assertNotIn(u'collective.nitf', tiles)

    # FIXME: https://github.com/collective/collective.nitf/issues/168
    @unittest.skipIf(DEXTERITY_ONLY, 'Known failure')
    def test_link_workflow_restored(self):
        workflow_tool = self.portal.portal_workflow
        chain = workflow_tool.getChainForPortalType('Link')
        # default binding
        self.assertEqual(len(chain), 0)
